/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef DOT_FILE_PRINTER_H
#define DOT_FILE_PRINTER_H

#include <string>
#include <vector>
#include <map>
#include <set>
#include <fstream>
#include <sstream>

#include "Transaction.h"
#include "State.h"
#include "Loop.h"
#include "util.h"
#include "global_vars.h"

using namespace std;

class Dot_File_Printer {

 public:

  static const int num_colors;
  static const int num_shapes;
  static const int num_styles;
  static const int num_edge_colors;

  static const string colors[18];
  static const string shapes[5];
  static const string styles[4];
  static const string edge_colors[12];
        
  bool show_hex;
  bool strip_leading_zeros;

  Dot_File_Printer();
  ~Dot_File_Printer();

  void print_state_graph(const string& file_name, const map<int,State>& states) const;
  void print_transaction_graph(const string& file_name, const vector<Transaction>& transactions,
                               const map<int,State>& states) const;
  string nice_key(const State& state) const;
  string nice_label(const State& state) const;
  vector<State> find_transaction_states(const Transaction& t, const map<int,State>& states) const;
  string transaction_sub_graph_header(const Transaction& t, int sub_graph_ID) const;
  string bubbles(const vector<State>& states, int transaction_ID) const;
  string bubble(const State& state, int transaction_ID, bool top_state, const string& label) const;
  string edge(const State& from_state,
              const State& to_state,
              int transaction_ID,
              set<pair<int,int> >& seen_edges,
              const string& color,
              const string& times_string,
              const float& weight,
              const float& max_weight=0,
              const float& min_weight=0) const;
  string get_edge_label(const State& from_state, const State& to_state, const float& weight) const;
  string loop_times(const Loop& loop) const;
  string transaction_sub_graph(const Transaction& t, const map<int,State>& states, int transaction_ID) const;
  string graph_header(const string& file_name, const string& key) const;
  string table(const string& contents) const;
  string table_row(const string& contents) const;
  string table_cell(const string& contents) const;
  string font_color(const string& color, const string& contents) const;
};

#endif //DOT_FILE_PRINTER_H
