/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#include <iostream>

using namespace std;

#ifndef LOGICVALUE_H
#define LOGICVALUE_H

enum fourValues_t { Log_0, Log_1, Log_X, Log_Z };
        
class FourValuedLogic {
 public:
  static const fourValues_t andTable[4][4];
  static const fourValues_t orTable[4][4];
  static const fourValues_t xorTable[4][4];
  static const fourValues_t notTable[4];
  static const char charTable[4];
  static const int intTable[4];
        
 FourValuedLogic():_val(Log_X){}
 FourValuedLogic(const fourValues_t &tv):_val(tv){}
 FourValuedLogic(const FourValuedLogic &tvl):_val(tvl._val){}
  FourValuedLogic(const char c) {
    switch (c) {
    case '0':
      _val = Log_0;
      break;
    case '1':
      _val = Log_1;
      break;
    case 'X':
      _val = Log_X;
      break;
    case 'Z':
      _val = Log_Z;
      break;
    default:
      _val = Log_X;
    }
  }
  ~FourValuedLogic() {}
        
  bool isX()
  { return _val==Log_X; }
  int toInt() const
  { return intTable[_val]; };
        
  const FourValuedLogic& operator=(const FourValuedLogic &tvl)
    { _val=tvl._val; return *this; }
  bool operator==(const FourValuedLogic &tvl) const
  { return _val==tvl._val; }
  bool operator!=(const FourValuedLogic &tvl) const
  { return _val!=tvl._val; }
  FourValuedLogic operator~() const
  { return FourValuedLogic(notTable[_val]); }
  FourValuedLogic& negate()
    { _val = notTable[_val]; return *this; }
  FourValuedLogic operator&(const FourValuedLogic &v) const
  { return FourValuedLogic(andTable[_val][v._val]); }
  const FourValuedLogic& operator&=(const FourValuedLogic &v)
  { _val = andTable[_val][v._val]; return *this; }
  FourValuedLogic operator|(const FourValuedLogic &v) const
  { return FourValuedLogic(orTable[_val][v._val]); }
  const FourValuedLogic& operator|=(const FourValuedLogic &v)
  { _val = orTable[_val][v._val]; return *this; }
  FourValuedLogic operator^(const FourValuedLogic &v) const
  { return FourValuedLogic(xorTable[_val][v._val]); }
  const FourValuedLogic& operator^=(const FourValuedLogic &v)
  { _val = xorTable[_val][v._val]; return *this; }

  friend ostream& operator<<(ostream &os, const FourValuedLogic &v)
  { os << v.charTable[v._val]; return os; }

 protected:
  fourValues_t _val;
};
        
// extern const FourValuedLogic FOUR_VALUED_LOGIC_0;
// extern const FourValuedLogic FOUR_VALUED_LOGIC_1;
// extern const FourValuedLogic FOUR_VALUED_LOGIC_X;
// extern const FourValuedLogic FOUR_VALUED_LOGIC_Z;

#endif //LOGICVALUE_H
