/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#include <cctype>
#include <cstdio>

#include "FourValuedVector.h"

// const FourValuedVector VECTOR_0(FOUR_VALUED_LOGIC_0);
// const FourValuedVector VECTOR_1(FOUR_VALUED_LOGIC_1);

FourValuedVector::FourValuedVector() {}

FourValuedVector::FourValuedVector(const FourValuedLogic& val) {
  push_back(val);
}

FourValuedVector::FourValuedVector(const string& val) {
  for(unsigned int i = 0; i < val.size(); ++i) {
    push_back(FourValuedLogic(toupper(val[i])));
  }
}

FourValuedVector::FourValuedVector(const string& val,int num_reps) {
  for(int j = 0; j < num_reps; ++j) {
    for(unsigned int i = 0; i < val.size(); ++i) {
      push_back(FourValuedLogic(toupper(val[i])));
    }
  }
}

FourValuedVector::~FourValuedVector() {}
        
void FourValuedVector::print() {
  for(FourValuedVector::iterator it = begin(); it != end(); ++it)
    cout << *it;
}

string FourValuedVector::get_string_w_dont_cares(bool show_hex, bool strip_leading_zeros) const {
  string result;
        
  if(!show_hex)
    {
      for(FourValuedVector::const_iterator it = begin(); it != end(); ++it)
        if(*it == FourValuedLogic('0'))
          result += "0";
        else if(*it == FourValuedLogic('1'))
          result += "1";
        else
          result += "-";
    }
  else
    {
        
      int pos = 0;
      int val = 0;
      bool dontcare = false;
                
      // iterate from LSB to MSB
      for(FourValuedVector::const_reverse_iterator it = rbegin(); it != rend(); ++it)
        {
          if(*it == FourValuedLogic('0'))
            val += 0;
          else if(*it == FourValuedLogic('1'))
            val += (1 << pos);
          else
            dontcare = true;
                
          pos++;
                        
          if(pos == 4) 
            {
              if (dontcare) result = "-" + result;
              else
                {
                  // convert value to hex and add to result
                  char hex_str[2];
                  sprintf(hex_str, "%x", val);
                                        
                  result = string(hex_str) + result;
                }
                                
              val = 0;
              pos = 0;
              dontcare = false;
            }
        }
                
      // # of bits may not be a multiple of 4, print hex for leading bits
      if(pos > 0)
        {
          if (dontcare) result = "-" + result;
          else
            {
              // convert value to hex and add to result
              char hex_str[2];
              sprintf(hex_str, "%x", val);
                                        
              result = string(hex_str) + result;
            }
        }
    }
        
  if(strip_leading_zeros)
    {
      int first_nonzero = result.find_first_not_of("0");
                
      // if all zeros, output is simply "0"
      if(first_nonzero == (int)string::npos)
        result = "0";
      else
        result = result.substr(first_nonzero);
    }
        
  return result;
}

void FourValuedVector::negate() {
  for(FourValuedVector::iterator it = begin(); it != end(); ++it)
    it->negate();
}

FourValuedVector FourValuedVector::operator~() const {
  FourValuedVector tmp = *this;
  for(FourValuedVector::iterator it = tmp.begin(); it != tmp.end(); ++it)
    it->negate();
  return tmp;
}

bool FourValuedVector::operator==(const FourValuedVector& v) const {
  if(size() != v.size()) {
    return false;
  }
  for(FourValuedVector::const_iterator it = begin(), it2 = v.begin(); it != end(); ++it, ++it2) {
    if(*it != *it2) {
      return false;
    }
  }
  return true;
}


ostream& operator<<(ostream &os, const FourValuedVector &v) {

  for(FourValuedVector::const_iterator it = v.begin(); it != v.end(); ++it)
    os << *it;
  return os;
}
