/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef MODULE_H
#define MODULE_H

#include <vector>
#include <string>
#include <map>
#include <iostream>

#include "Module_Child.h"
#include "Directions.h"
#include "util.h"
#include "global_vars.h"


using namespace std;


typedef map<string,Module_Child> children_t;
typedef vector<pair<string,vector<string> > > interfaces_t;

class Module
{
 public:
        
  string type;
  map<string,int> inputs, outputs, internals;
  vector<pair<string,string> > sub_modules; //(type,name)
  children_t children;

  Module();
  ~Module();

  void clear();
  bool has_child(const string& child_name) const;
  signal_direction_t get_direction(const string& sig_name) const;
  string get_child_type(const string& child_name) const;
  vector<string> get_interface(const string& module_name, vector<string>& signal_name,
                               const string& main_module) const;
  vector<string> get_output_signal_names(const vector<string>& signal_names) const;
        
  friend ostream& operator<<(ostream &os, const Module &s);

 protected:
};

#endif // MODULE_H
