/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef MULTI_STREAM_H
#define MULTI_STREAM_H

#include <iostream>
#include <ostream>
#include <fstream>
#include <vector>

using namespace std;

class Multi_Stream : public ostream
{
 private:
  int mode;
  vector<ostream*> streams;
  
public:
  Multi_Stream();

  int AddStream(ostream* stream);

  void SetMode(int flags);
  
  void ActivateStream(int index);
  
  void DeactivateStream(int index);
  
  // template to handle most types
        template<class T>
  Multi_Stream& operator<<(T obj)
          {
    int flags = 1;
    for(unsigned int i=0;i<streams.size();i++)
    {
      if(flags & mode)
        (*streams[i])<<obj;
      flags <<= 1;
    }
    return *this;
  }
  
  
        // handle i/o manipulators (e.g. endl)
        Multi_Stream& operator<<(std::basic_ostream<char, std::char_traits<char> >& 
                                 (*)(std::basic_ostream<char, std::char_traits<char> >&));

        // handle C strings
        Multi_Stream& operator<<(const char* str)
          {  
            int flags = 1;
    for(unsigned int i=0;i<streams.size();i++)
    {
      if(flags & mode)
        (*streams[i]) << str;
      flags <<= 1;
    }
    return *this;
          }
        
};

#endif
