/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef SIGNAL_H
#define SIGNAL_H

#include <vector>
#include <string>
#include <iostream>

#include "FourValuedVector.h"
#include "Directions.h"
#include "util.h"
#include "global_vars.h"

using namespace std;

typedef FourValuedVector value_t;
typedef vector<pair<unsigned long,value_t> > values_t;

class Signal
{
 public:
        
  string full_name; //The whole thing -- from the top-level module on down
  string name;      //Just the local name (no "."'s)
  string module_name; //Just the immediate module name
  string parent_name; //Name of the parent module
  int num_bits;
  values_t values;
  signal_direction_t direction;

  values_t::const_iterator cur_it, next_it, max_it;

  Signal();
  Signal(const string& full_name_in, const string& name_in, const string& module_name_in,
         const string& parent_name_in, int num_bits_in);
  Signal(string full_name_in, int num_bits_in);
  ~Signal();

  vector<Signal> get_single_bits() const;
  unsigned long max_time() const;
  void remove_glitches();
  void setup_iterators();
  bool goto_next(unsigned long min_time);
  unsigned long next_time(unsigned long last_time) const;
  unsigned long next_pos_edge(unsigned long last_time);
  unsigned long prev_pos_edge(unsigned long last_time);
  FourValuedVector cur_value() const;
  void replace_values(const values_t& new_values);
  void clear_values();
  bool not_internal();
        
  friend ostream& operator<<(ostream &os, const Signal &s);

 protected:
};

#endif // SIGNAL_H
