/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef SIGNAL_GROUP_H
#define SIGNAL_GROUP_H

#include <map>
#include <string>
#include <iostream>

#include "Signal.h"

using namespace std;

typedef map<string,Signal> signals_t;

class Signal_Group
{
 public:
        
  //    string name;
  bool have_clock, have_reset;

  Signal_Group();
  ~Signal_Group();

  void configure(const string& clock_name_in, const string& reset_name_in, bool sync_mode_in);
  void add_signal(const Signal& s);
  void prepare_to_find_states(unsigned long max_time);
  void replace_values(const string& signal_name, const values_t& values);
  int num_signals() const;
  void restart();
  void goto_next(unsigned long min_next_time);
  unsigned long first_min_next_time();
  unsigned long min_next_time(unsigned long last_time);
  unsigned long max_time() const;
  void add_ends(unsigned long max_t);
  map<string,FourValuedVector> get_values() const;
  bool all_done() const;

 protected:
  signals_t signals;
  string clock_name, reset_name;
  Signal clock_signal, reset_signal;
  bool sync_mode;
  unsigned int num_done;
};

#endif // SIGNAL_GROUP_H
