/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef STATE_H
#define STATE_H

#include <vector>
#include <string>
#include <map>
#include <set>

#include "FourValuedVector.h"
#include "global_defs.h"
#include "Transaction.h"
#include "global_vars.h"

typedef map<string,FourValuedVector> state_values_t;
typedef pair<int,long> next_state_t;

class State
{
 public:
  int ID;
  state_values_t values;
  //vector<int> nexts;
  vector<next_state_t> nexts;
  long total_weight;

  State();
  State(int ID_in,const state_values_t& values_in);
  virtual ~State();

  void add_next(int next_ID);
  void remove_signal(const string& signal_name);
  bool all_constant() const;
  int get_num_signal_bits(const string& signal_names) const;
  int get_num_signal_bits(const vector<string>& signal_names) const;
  FourValuedVector get_signal_value(const string& signal_name) const;
  vector<string> get_signal_names() const;
  map<string,FourValuedVector> get_name_value_map(const vector<string>& signal_names) const;
  string get_signal_values_string(const vector<string>& signal_names) const;
  vector<FourValuedVector> get_signal_values(const vector<string>& signal_names) const;
  bool operator==(const State& s) const;

  friend ostream& operator<<(ostream &os, const State &s);
        
 private:
};


#endif // STATE_H
