/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef SYSTEM_H
#define SYSTEM_H

#include <vector>
#include <string>
#include <map>
#include <set>
#include <fstream>
#include <math.h>
#include <iomanip>

#include "Config_Parser.h"
#include "Verilog_Parser.h"
#include "VCD_Parser.h"
#include "Module.h"
#include "Module_Child.h"
#include "State.h"
#include "Signal_Group.h"
#include "Transaction.h"
#include "Dot_File_Printer.h"
#include "Verilog_Printer.h"
#include "util.h"
#include "global_vars.h"

typedef map<string,Module> modules_t;
typedef map<int,State> states_t; 
typedef map<string,FourValuedVector> constants_t;

class System
{
 public:
  string config_file_name;

  //Config info gathered in main.c
  bool found_sis;
  bool found_abc;

  //From config file
  string output_file_base, top_name;
  int max_lines;
  string reset_name, clock_name;
  bool transaction_mode, sync_mode, signal_mode, all_signals, start_mode;
  vector<string> start_signal_names, start_triggers, verilog_f_names;
  string trace_file_name;
  int max_bits;  //max bits for a signal to be considered
  string main_module;
  set<string> signals_to_analyze, modules_to_analyze;
  bool loopback_edges;
  bool show_hex;
  vector<string> ignore_states;
  bool strip_leading_zeros;
        
  //From reading verilog
  modules_t modules;
  map<string,string> module_name_to_type_translation;
  map<string,string> defined_constants;

  //From reading trace file
  VCD_Parser vcd_parser;
  bool done_parsing;

  states_t states;
  vector<int> state_sequence;
  vector<unsigned long long> time_sequence;
  vector<Transaction> transactions; //number of occurrences, sequence
  vector<Transaction> approved_transactions;
  constants_t constants;
  unsigned int last_ID;
  unsigned long last_time;

  Signal_Group signals;

  Dot_File_Printer df_printer;
        
  System();
  System(const string& config_file_name_in);
  System(const System& sys);
  virtual ~System();

  bool read_config_file();
  bool parse_verilog();
  bool parse_beginning_of_trace_file();
  bool parse_more_of_trace_file();
  bool re_translate(Signal& s);

  string get_interface(const string& parent_name, const string& child_name,
                       string& signal_name, const string& main_module) const;
  void add_state(map<string,FourValuedVector> values, unsigned long cur_time);
  void find_states(); 
  void find_constants_in_set(const vector<int>& state_IDs,
                             map<string,FourValuedVector>& constants);
  void find_constants();
  bool start_transition(int last_state,int cur_state,const vector<string>& start_signals,
                        const vector<string>& start_triggers);
  void match_transactions();
  void find_initial_transactions(bool start_mode, const vector<string>& start_signal,
                                 const vector<string>& start_trigger);
  //    void analyze_transactions();
  vector<int> find_other_transaction_ends();
  void find_transactions();
  void find_transaction_constants();
  void print_state_diagram() const;
  bool print_dot_file(const string& dot_file_name = "", bool all_transactions = false);
  void print_verilog();
  void clean_files();
  void print_state_sequence();
  void find_correlations();
  void print_summary();
  friend ostream& operator<<(ostream &os, const System &s);
        
 private:
};


#endif // SYSTEM_H
