/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef TRANSACTION_H
#define TRANSACTION_H

#include <vector>
#include <string>
#include <set>
#include <iostream>
#include <map>

#include "Loop.h"
#include "global_vars.h"

using namespace std;

//typedef pair<int,vector<int> > transactions_t; //number of occurrences, sequence

class Transaction
{
 public:
  //    transactions_t transaction; //number of occurrences, sequence
  int num_occurrences;
  vector<int> sequence;
  vector<Loop> loops;
  set<string> constants;
  unsigned long long time_first_seen;
        
  Transaction();
  Transaction(int num_occurrences_in, const vector<int>& sequence_in, unsigned long long time_first_seen_in);
  virtual ~Transaction();

  int size() const;
  void add_state(int ID);
  void clear();
  void add_constant(const string& constant);
  bool is_constant(const string& name) const;
  void expand_loop(vector<int>::iterator& loop_begin, vector<int>::iterator& loop_end,
                   const vector<int>& tmp_sequence);
  void remove_inner_loops(Loop& cur_loop);
  void find_loops();
  vector<int> get_all_IDs() const;
  vector<int>::const_iterator find_end(const Transaction& t) const;
  bool matches(const Transaction& t) const;
  bool has_loop(const Loop& l) const;
  void combine(const Transaction& t);
  vector<pair<int,int> > get_transitions() const;
        
  bool operator==(const Transaction& t) const;

  friend ostream& operator<<(ostream &os, const Transaction &t);
 private:
};

#endif // TRANSACTION_H
