/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef VCD_PARSER_H
#define VCD_PARSER_H

#include <assert.h>
#include <string>
#include <vector>
#include <map>
#include <set>
#include <iostream>
#include <fstream>

#include "global_defs.h"
#include "Module.h"
#include "global_vars.h"

class VCD_Parser {

 public:

  int max_lines;
  int max_bits;

  string vcd_file_name;
  ifstream vcd_file;
  set<string> modules_to_analyze;
  string clock_name;
  bool signal_mode;
  bool all_signals;
  set<string> signals_to_analyze;
  bool sync_mode;

  vector<string> scope;
  vector<pair<string,string> > initial_var_translation;
  vector<int> initial_num_bits;
  map<string,string> var_translation;
  data_t data;
  dup_t duplicates;
  vector<string> words;
  vector<string> ignore_states;

  bool done_parsing;
  unsigned long time;
  int total_lines, lines_of_this_chunk;
  bool last_time_step;
        
  VCD_Parser();
  ~VCD_Parser();
        
  void initialize(const int max_lines_in, const int max_bits_in, const string& vcd_file_name_in,
                  const set<string>& modules_to_analyze_in, const string& clock_name_in,
                  bool signal_mode_in, bool all_signals_in,
                  const set<string>& signals_to_analyze_in, bool sync_mode_in,
                  const vector<string>& ignore_states_in);
  bool parseBegin();
  bool parseContinue(const modules_t& modules,
                     const map<string,string>& module_name_to_type_translation);
                                        
 private:

  bool find_next_module_scope();
  void read_vars();
  void readDefinitions();
  bool readValues(const modules_t& modules,
                  const map<string,string>& module_name_to_type_translation);
  bool parse_line();
  //    bool want_to_analyze(const string& var_name);
};

#endif //VCD_PARSER_H
