/******************************************************************************
    Inferno : An automatic semantic information extractor
    Copyright (C) 2006-2010  University of Michigan
    http://www.eecs.umich.edu/inferno

    Contributors include Andrew DeOrio, Adam Bauserman, 
    Valeria Bertacco and Beth Isaksen.    

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301  USA

    Andrew DeOrio <awdeorio@umich.edu>
    Valeria Bertacco <valeria@umich.edu>

    University of Michigan
    Electrical Engineering and Computer Science Dept.
    2260 Hayward St.
    Ann Arbor, MI 48109-2121

******************************************************************************/

#ifndef VERILOG_PARSER_H
#define VERILOG_PARSER_H

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <fstream>
#include <cassert>

#include "global_defs.h"
#include "Module.h"
#include "global_vars.h"

class Verilog_Parser {

 public:

  vector<string> file_names;
  string top_name;
        
  Verilog_Parser();
  Verilog_Parser(const vector<string>& file_names_in, const string& top_name_in);
  ~Verilog_Parser();

  bool parse(modules_t& modules, map<string,string>& module_name_to_type_translation,
             map<string,string>& defined_constants);

 private:
  int parse_num(string unparsed);
  int get_num_bits(string range);
  void add_signal_to_module(const vector<string>& words, Module& m);
  bool readModuleDefs(ifstream& inFile, modules_t& modules, map<string,string>& defined_constants);
  bool readModuleInsts(ifstream& inFile, modules_t& modules, map<string,string>& module_name_to_type_translation,
                       map<string,string>& defined_constants);
        
};

#endif //VERILOG_PARSER_H
