/******************************************************************************
    Inferno : An automatic semantic information extractor
    Copyright (C) 2006-2010  University of Michigan
    http://www.eecs.umich.edu/inferno

    Contributors include Andrew DeOrio, Adam Bauserman, 
    Valeria Bertacco and Beth Isaksen.    

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301  USA

    Andrew DeOrio <awdeorio@umich.edu>
    Valeria Bertacco <valeria@umich.edu>

    University of Michigan
    Electrical Engineering and Computer Science Dept.
    2260 Hayward St.
    Ann Arbor, MI 48109-2121

******************************************************************************/

#ifndef VERILOG_PRINTER_H
#define VERILOG_PRINTER_H

#include <string>
#include <vector>
#include <set>
#include <fstream>

#include "State.h"
#include "Transaction.h"
#include "util.h"
#include "global_vars.h"

using namespace std;

class Verilog_Printer {

 public:

  map<int,State> states;
  map<string,FourValuedVector> constants;
  vector<string> constant_names;
  vector<Transaction> transactions;
  vector<string> signal_names;
  string output_file_base;
  bool transaction_mode;
        
  bool found_sis;
  bool found_abc;

  Verilog_Printer();
  Verilog_Printer(const map<int,State>& states_in, const map<string,FourValuedVector>& constants_in,
                  const vector<Transaction>& transactions_in,
                  const vector<string>& signal_names_in, const string& output_file_base_in,
                  bool transaction_mode_in);
  ~Verilog_Printer();

  void find_legal_states();
  void print_legal_state_checker_verilog();
  //    void find_legal_transitions();
  void find_legal_transitions_by_signal(const string& f_name_base,
                                        const string& signal_name);
  void find_legal_transitions_for_transaction(Transaction t, string f_name_base);
  void find_legal_ends(const set<int>& ends, string f_name_base);
  void find_legal_transitions_for_all();
  void print_legal_transitions_by_signal(const string& f_name_base, const string& signal_name);
  void print_legal_transition_verilog(const string& out_file_name);
  void print_legal_transition_all_verilog(const string & out_file_name);
  void print_legal_transition_checker_verilog();
  void assigns_from_eqn(const string& eqn_file_name, const string& uniquify_string,
                        vector<string>& inputs, vector<string>& outputs,
                        vector<string>& assigns, vector<string>& wires);
  void minimize_logic(string input_file, string output_file);
};

#endif //VERILOG_PRINTER_H
