/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#include "util.h"
#include <cstdlib>

//Test comment

int max(int n1, int n2) {
  if(n1 > n2)
    return n1;
  else
    return n2;
}

bool getLineNoComments(ifstream& inFile,string& line,
                       map<string,string>& defined_constants,
                       const string& singleLine, const string& start, const string& end ) {
  bool foundLine = true;
  line = "";
  string::size_type start_pos;
  string tmp_line;

  //    mout << "getLineNoComments\n";
  while(foundLine && line.size() == 0) {
    foundLine = getline(inFile,line);
    //          mout << "line: " << line << endl;
    line = line.substr(0,line.find(singleLine));
    //          mout << "without comment: '" << line << "'" << endl;
    if(line.size() > 0) {
      start_pos = line.find(start);
      if(start_pos != string::npos) {
        //                              mout << "multi-line comment encountered at " << start_pos << ".\n";
        tmp_line = line.substr(0,start_pos);
        //                              mout << "tmp_line: " << tmp_line << endl;
        while(line.find(end) == string::npos) {
          if(!getline(inFile,line))
            break;
        }
        line = line.substr(line.find(end));
        line = tmp_line + line;
      }
    }
    //          mout << "without multi-line comment: '" << line << "'" << endl;
    if(line.size() > 0) {
      //                        mout << "checking for defs, etc.\n";
      vector<string> words;
      splitString(line," \t",words);
      if(words.size() == 0) {
        break;
      }
      for(map<string,string>::iterator it = defined_constants.begin();
          it != defined_constants.end(); ++it) {
        if(line.find("`" + it->first) != string::npos) {
          int start = line.find("`" + it->first);
          int size = it->first.size();
          line.replace(start,size+1,it->second);
        }
      }
      //                        mout << "words[0]: " << words[0] << endl;
      if(words[0] == "`ifdef") {
        if(defined_constants.find(words[1]) != defined_constants.end()) {
          //++ifdef;
        }
        else {
          bool done = false;
          while(!done) {
            foundLine = getline(inFile,line);
            if(!foundLine) {
              done = true;
            }
            else {
              (splitString(line," \t",words));
              if(words[0] == "`else" || words[0] == "`endif")
                done = true;
            }
          }
        }
        line = "";
      }
      else if(words[0] == "`else") {
        bool done = false;
        while(!done) {
          foundLine = getline(inFile,line);
          if(!foundLine) {
            done = true;
          }
          else {
            (splitString(line," \t",words));
            if(words[0] == "`endif")
              done = true;
          }
        }
        line = "";
      }
      else if(words[0] == "`endif") {
        line = "";
      }
      else if(words[0] == "`define" || words[0] == "-DEFINE") {
        if(words.size() > 2)
          defined_constants[words[1]] = words[2];
        else
          defined_constants[words[1]] = "";
      }
    }
  }
  //    mout << "final line: " << line << endl;
  return line.size() != 0;
}

bool getWords(ifstream& inFile, vector<string>& words, bool stripComments, const string& delim) {
  map<string,string> empty_map;
  return getWords(inFile,words,stripComments,empty_map,delim);
}

bool getWords(ifstream& inFile, vector<string>& words, bool stripComments, map<string,string>& defined_constants,
              const string& delim) {
  string curLine;
  bool found = 0;
  words.clear();

  //    mout << "getWords, stripping comments? " << stripComments << endl;
  //skip blank lines
  while(!found) {
    if(stripComments) {
      if(!getLineNoComments(inFile,curLine,defined_constants))
        return false;
    }
    else {
      if(!getline(inFile,curLine))
        return false;
    }
    //          mout << "getWords, curLine: " << curLine << endl;
    splitString(curLine,delim,words);

    if(stripComments) {
      vector<string> tmp_words;
      unsigned int i = 0;
      while(i < words.size()) {
        //                      mout << "words[" << i << "]: " << words[i] << endl;
        if(words[i].find_first_of('[') != string::npos
           && words[i].find_first_of(']') == string::npos) {
          //                            mout << "unbalanced []: " << words[i] << endl;
          tmp_words.push_back(words[i]);
          ++i;
          string tmp;
          while(i < words.size()
                && words[i].find_first_of(']') == string::npos) {
            //                                  mout << "adding: " << words[i] << endl;
            tmp = tmp_words.back();
            tmp_words.pop_back();
            tmp_words.push_back(tmp + words[i]);
            ++i;
          }
          if(i < words.size()) {
            //                                  mout << "last one: " << words[i] << endl;
            tmp = tmp_words.back();
            tmp_words.pop_back();
            tmp_words.push_back(tmp + words[i]);
          }
        }
        else {
          tmp_words.push_back(words[i]);
        }
        ++i;
      }
      words = tmp_words;
    }
    found = words.size() > 0;
  }
  //    for(unsigned int i = 0; i < words.size(); ++i)
  //            mout << "words[" << i << "]: " << words[i] << " ";
  //    mout << endl;
  //    mout << "getWords, words[0]: " << words[0] <<
  //            " words.size: " << words.size() << "\n";
  return true;
}


void replace_nots(string& s) {
  string::size_type not_pos = s.find("!");
  while(not_pos != string::npos) {
    s.replace(not_pos,1,"~");
    not_pos = s.find("!");
  }
}

void replace_ands(string& s) {
  string::size_type and_pos = s.find("*");
  while(and_pos != string::npos) {
    s.replace(and_pos,1," & ");
    and_pos = s.find("*");
  }
}
void replace_ors(string& s) {
  string::size_type or_pos = s.find("+");
  while(or_pos != string::npos) {
    s.replace(or_pos,1," | ");
    or_pos = s.find("+");
  }
}

void replace_braces(string& s, const string& uniquify_string) {
  string::size_type open_brace_pos = s.find("[");
  while(open_brace_pos != string ::npos) {
    s.replace(open_brace_pos,1,uniquify_string + "expr");
    open_brace_pos = s.find("[");               
  }
  string::size_type close_brace_pos = s.find("]");
  while(close_brace_pos != string ::npos) {
    s.replace(close_brace_pos,1,"");
    close_brace_pos = s.find("]");              
  }
}

int splitString(const string& input, const string& delimiter, vector<string>& results)
{
  return splitString(input, delimiter, results, true);
}

int splitString(const string& input, const string& delimiter, vector<string>& results, bool trim_whitespace)
{
  int startPos = input.find_first_not_of(delimiter);
  int endPos = -1;
  int isize = input.size();
  
  vector<int> startPositions;
  vector<int> endPositions;

  //  mout << "startPos: " << startPos << "\n";
  results.clear();
  endPos = input.find_first_of(delimiter, startPos);
  //  mout << "endPos: " << endPos << "\n";

  while(endPos > 0)
    {
      startPositions.push_back(startPos);
      endPositions.push_back(endPos);
      startPos = input.find_first_not_of(delimiter,endPos);
      //        mout << "startPos: " << startPos << "\n";
      endPos = input.find_first_of(delimiter,startPos);
      //        mout << "endPos: " << endPos << "\n";
    }
  if(startPos < isize && startPos >= 0) {
    startPositions.push_back(startPos);
    endPositions.push_back(isize);
  }

  for( int i=0; i < (int) startPositions.size(); i++ )
    {
      string s;
      s = input.substr(startPositions.at(i), \
                       endPositions.at(i) - startPositions.at(i));
    
      if(trim_whitespace) trim(s);
    
      if( s.size() > 0 )
        {
          results.push_back(s);
        }
    }
  return results.size();
}

string concat_string(vector<string>& inputs, const string& delimiter) {
  string result;
  for(vector<string>::iterator it = inputs.begin(); it != inputs.end(); ++it) {
    if(it != inputs.begin())
      result += delimiter;
    result += *it;
  }
  return result;
}

string substr_before_dot(const string& in) {
  // given a string in form "s1.s2" returns s1
  return in.substr(0,in.find(".",0));
}

string substr_after_dot(const string& in) {
  // given a string in form "s1.s2" returns s2
  return in.substr(in.find(".",0) + 1);
}

bool parse_signal_name(const string& full_name, string& name, string& module_name,
                       string& parent_name, string& full_module_name) {
  vector<string> name_pieces;
  int num_name_pieces = splitString(full_name,".",name_pieces);
  if(num_name_pieces < 2) {
    mout << "Error: signal name encountered with no module name while "
         << "reading variable translation from vcd file: " << full_name << endl;
    return false;
  }
  else {
    name = name_pieces[name_pieces.size() - 1];
    module_name = name_pieces[name_pieces.size() - 2];
    if(num_name_pieces == 2) {
      parent_name = module_name; //exceptional case
      full_module_name = module_name;
    }
    else {
      parent_name = name_pieces[name_pieces.size() - 3];
      full_module_name = full_name.substr(0, full_name.find_last_of("."));
    }
  }
  return true;
}

vector<string> get_short_names(const vector<string>& names, const string& suffix1, const string& suffix2) {
  vector<string> results;
  for(vector<string>::const_iterator it = names.begin(); it != names.end(); ++it) {
    string::size_type dot_pos = it->rfind(".");
    //  mout << "dot_pos: " << dot_pos << endl;
    string tmp = *it;
    tmp.replace(dot_pos,1,"_");
    dot_pos = it->rfind(".",dot_pos - 1);
    tmp = tmp.substr(dot_pos + 1);
    results.push_back(tmp);
    //  mout << "dot_pos: " << dot_pos << endl;
  }
  return get_with_suffixes(results,suffix1, suffix2);
}

vector<string> get_with_suffixes(const vector<string>& names, const string& suffix1, const string& suffix2) {
  vector<string> results;
                                                           
  for(vector<string>::const_iterator it = names.begin(); it != names.end(); ++it) {
    string tmp = *it;
    if(tmp.find("[") != string::npos) 
      tmp = tmp.substr(0,tmp.find("[")) + suffix1 + tmp.substr(tmp.find("[")) + suffix2;
    else
      tmp = tmp + suffix1 + suffix2;
    results.push_back(tmp);
  }
  return results;
}

vector<string> get_with_prefixes(const vector<string>& names, const string& prefix) {
  vector<string> results;
                                                           
  for(vector<string>::const_iterator it = names.begin(); it != names.end(); ++it) {
    string tmp = *it;
    tmp = prefix + tmp;
    results.push_back(tmp);
  }
  return results;
}

string get_with_delim(const vector<string>& names, const string& delim) {
  string results;
                                                           
  for(vector<string>::const_iterator it = names.begin(); it != names.end(); ++it) {
    if(it != names.begin())
      results += delim;
    results += *it;
  }
  return results;
}

string get_short_names_as_string(const vector<string>& names, const string& prefix, const string& suffix1,
                                 const string& suffix2, const string& delim) {
  string results;
  vector<string> short_names = get_short_names(names,suffix1,suffix2);
  return get_with_delim(short_names,delim);
}

string get_names_as_string(const vector<string>& names, const string& prefix, const string& suffix1,
                           const string& suffix2, const string& delim) {
  return get_with_delim(get_with_prefixes(get_with_suffixes(names,suffix1,suffix2),prefix),delim);
}

string get_short_name(const string& name) {
  string::size_type dot_pos = name.rfind(".");
  //    mout << "dot_pos: " << dot_pos << endl;
  string tmp = name;
  tmp.replace(dot_pos,1,"_");
  dot_pos = tmp.rfind(".",dot_pos - 1);
  tmp = tmp.substr(dot_pos + 1);
        
  return tmp;
}

string get_really_short_name(const string& name) {
  string::size_type dot_pos = name.rfind(".");
  //    mout << "dot_pos: " << dot_pos << endl;
  return name.substr(dot_pos + 1);
}

string get_short_module_name(const string& name) {
        
  // given a module name, return the name in the format 'parent.module'
        
  // find last dot
  string::size_type last_dot =  name.rfind(".");
        
  // rare case: only one module (top-level)
  if(last_dot == string::npos) return name;
        
  //find the second to last dot
  last_dot = name.rfind(".", last_dot - 1);
        
  // is the parent module the top-level?
  if(last_dot == string::npos) return name;
        
  // strip everything before the second-to-last dot
  return name.substr(last_dot + 1);
        
}

string get_checker_warning() {

  string warning = "// synopsys translate_off\n";
  warning += "  always @(fail) begin\n";
  warning += "    if (fail == 1'b1) $display(\"Error: checker failure: %m\");\n";
  warning += "  end\n";
  warning += "// synopsys translate_on\n";

  return warning;
}

string make_vector_string(const string& word, const string& prefix, const string& suffix1,
                          const string& suffix2, const string& delim,
                          int min, int max) {
  vector<string> results_vector;
  for(int i = min; i <= max; ++i) {
    string cur_word = word;
    string::size_type pos = 0;
    while((pos = cur_word.find("[",pos)) != string::npos) {
      cur_word.replace(pos,2,"[" + itoa(i) + "]");
      ++pos;
    }
    results_vector.push_back(cur_word);
  }
  return get_names_as_string(results_vector, prefix, suffix1, suffix2, delim);
}

vector<string> square_to_pointy_braces(const vector<string> & inputs) {
  vector<string> results;
  for(vector<string>::const_iterator it = inputs.begin(); it != inputs.end(); ++it) {
    results.push_back(square_to_pointy_braces(*it));
  }
  return results;
}

vector<string> pointy_to_square_braces(const vector<string> & inputs) {
  vector<string> results;
  for(vector<string>::const_iterator it = inputs.begin(); it != inputs.end(); ++it)
    results.push_back(pointy_to_square_braces(*it));
  return results;
}

string square_to_pointy_braces(const string & input) {
  string::size_type pos = 0;
  string result = input;
  //    mout << "square to pointy, before: " << result << endl;
  while((pos = result.find("[")) != string::npos) {
    //          mout << "pos: " << pos << endl;
    result.replace(pos,1,"<");
  }
  pos = 0;
  while((pos = result.find("]")) != string::npos) {
    //          mout << "pos: " << pos << endl;
    result.replace(pos,1,">");
  }
  //    mout << "square to pointy, after: " << result << endl;
  return result;
}

string pointy_to_square_braces(const string & input) {
  string::size_type pos = 0;
  string result = input;
  while((pos = result.find("<")) != string::npos) {
    result.replace(pos,1,"[");
  }
  pos = 0;
  while((pos = result.find(">")) != string::npos) {
    result.replace(pos,1,"]");
  }
  return result;
}

string format_regs(const vector<string>& words, const string& suffix) {
  return format_vector("   reg %s;\n",get_with_suffixes(words,suffix));
}

string format_inputs(const vector<string>& words) {
  return format_vector("   input %s;\n",words);
}

string format_wires(const vector<string>& words) {
  return format_vector("   wire %s;\n",words);
}

string format_assigns(const vector<string>& words) {
  return format_vector("   assign %s;\n",words);
}

// string format_vector(const string& format, const vector<string>& words1, const vector<string>& words2,
//                                       const string& delim) {
//      string result;
        
//      for(vector<string>::const_iterator it = words.begin(); it != words.end(); ++it) {
//              string::size_type pos = 0;
//              string cur_word = *it;
//              string cur_result = format;
//              while((pos = cur_result.find("%s1")) != string::npos) {
//                      cur_result = cur_result.replace(pos,2,cur_word);
//              }
//              pos = 0;
//              while((pos = cur_result.find("%s2")) != string::npos) {
//                      cur_result = cur_result.replace(pos,2,cur_word);
//              }
//              if(it != words.begin())
//                      result += delim;
//              result += cur_result;
//      }
//      return result;
// }

string format_vector(const string& format, const vector<string>& words, const string& delim) {
  string result;
        
  for(vector<string>::const_iterator it = words.begin(); it != words.end(); ++it) {
    string::size_type pos = 0;
    string cur_word = *it;
    string cur_result = format;
    while((pos = cur_result.find("%s")) != string::npos) {
      cur_result = cur_result.replace(pos,2,cur_word);
    }
    if(it != words.begin())
      result += delim;
    result += cur_result;
  }
  return result;
}

unsigned long atoi(string s) {
  char * pEnd;
  return strtoul(s.c_str(),&pEnd,0);
}

string itoa(int i) {
  char buf[512];
  sprintf(buf,"%d",i);
  return buf;
}

string itoa(unsigned long long int i) {
  char buf[512];
  sprintf(buf,"%llu",i);
  return buf;
}

// trim all whitespace from a string (spaces, tabs, and CR/LF)
void trim(string& str)
{
  string::size_type pos = str.find_last_not_of(" \t\r\n");
  if(pos != string::npos) {
    str.erase(pos + 1);
    pos = str.find_first_not_of(" \t\r\n");
    if(pos != string::npos) str.erase(0, pos);
  }
  else
    str.erase(str.begin(), str.end());
}


bool find_in_path(string cmd)
{
  string find_cmd = "which " + cmd + " 2> /dev/null";
  char path[BUFSIZ];
  FILE *ptr;

  if ((ptr = popen(find_cmd.c_str(), "r")) != NULL) {
    while (fgets(path, BUFSIZ, ptr) != NULL) {
      //wait while we get chars from command
      //printf("%s", buf);
    }
    pclose(ptr);
  }
  else {
    return 0;
  }

  if (!string(path).length() ||
      (string(path).find("no " + cmd,0) != string::npos)) {
    return 0;
  }
  else {
    return 1;
  }
}

