/**************************************************************************
 *   Inferno : An automatic semantic information extractor                 *
 *   Copyright (C) 2006-2010  University of Michigan                       *
 *   http://www.eecs.umich.edu/inferno                                     *
 *                                                                         *
 *   Contributors include Andrew DeOrio, Adam Bauserman,                   *
 *   Valeria Bertacco and Beth Isaksen.                                    *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
 *   02110-1301  USA                                                       *
 *                                                                         *
 *   Andrew DeOrio <awdeorio@umich.edu>                                    *
 *   Valeria Bertacco <valeria@umich.edu>                                  *
 *                                                                         *
 *   University of Michigan                                                *
 *   Electrical Engineering and Computer Science Dept.                     *
 *   2260 Hayward St.                                                      *
 *   Ann Arbor, MI 48109-2121                                              *
 **************************************************************************/

#ifndef UTIL_H
#define UTIL_H

//utility functions

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <fstream>

#include "global_vars.h"

using namespace std;

int max(int n1, int n2);

bool getLineNoComments(ifstream& inFile,string& line,
                       map<string,string>& defined_constants,
                       const string& singleLine = "//",const string& start = "/*", const string& end = "*/");
bool getWords(ifstream& inFile, vector<string>& words,bool stripComments,
              map<string,string>& defined_constants,
              const string& delim = " \t(),;");
bool getWords(ifstream& inFile, vector<string>& words,bool stripComments,
              const string& delim = " \t(),;");

void replace_nots(string& s);
void replace_ands(string& s);
void replace_ors(string& s);
void replace_braces(string& s, const string& uniquify_string);
int splitString(const string& input, const string& delimiter, 
                vector<string>& results);
int splitString(const string& input, const string& delimiter, 
                vector<string>& results, bool trim_whitespace);
string concat_string(vector<string>& inputs, const string& delimiter);
string substr_before_dot(const string& in);
string substr_after_dot(const string& in);
string itoa(int i);
string itoa(unsigned long long int i);
unsigned long atoi(string s);
void trim(string& str);

bool parse_signal_name(const string& full_name, string& name, string& module_name,
                       string& parent_name, string& full_module_name);

vector<string> get_with_suffixes(const vector<string>& names, const string& suffix1, const string& suffix2="");
vector<string> get_with_prefixes(const vector<string>& names, const string& prefix);
vector<string> get_short_names(const vector<string>& names, const string& suffix1="", const string& suffix2="");
string get_with_delim(const vector<string>& names, const string& delim);
string get_short_names_as_string(const vector<string>& names, const string& prefix="",
                                 const string& suffix1="", const string& suffix2="", const string& delim=" ");
string get_names_as_string(const vector<string>& names, const string& prefix="",
                           const string& suffix1="", const string& suffix2="", const string& delim=" ");
string make_vector_string(const string& word, const string& prefix, const string& suffix1, const string& suffix2,
                          const string& delim,
                          int min, int max);
string get_short_name(const string& name);
string get_really_short_name(const string& name);
string get_short_module_name(const string& name);
string get_checker_warning();
string format_vector(const string& format, const vector<string>& words, const string& delim="");
string format_regs(const vector<string>& words, const string& suffix);
string format_inputs(const vector<string>& words);
string format_wires(const vector<string>& words);
string format_assigns(const vector<string>& words);
string square_to_pointy_braces(const string & input);
string pointy_to_square_braces(const string & input);
vector<string> square_to_pointy_braces(const vector<string> & inputs);
vector<string> pointy_to_square_braces(const vector<string> & inputs);

bool find_in_path(string cmd);

#endif // UTIL_H

